<?php

namespace App\Service;

use App\Exception;

class ConsumerService {
    
    private $util;
    private $consumerFactory;
    private $branchFactory;

    public function __construct (\App\Util\Util $util, 
                                 \App\Factory\ConsumerFactory $consumerFactory,
                                 \App\Factory\BranchFactory $branchFactory) {
        $this->util            = $util;
        $this->consumerFactory = $consumerFactory;
        $this->branchFactory   = $branchFactory;
    }
    
    public function processConsumer ($generalParams, $orderData) {
        
        $consumer          = $this->consumerFactory->getStandardConsumer($generalParams["CDFILIAL"]);
        $branchAddressInfo = $this->branchFactory->getAddressInfo($generalParams["CDFILIAL"]);
        $address           = $this->processAddressInformation($orderData, $branchAddressInfo, $generalParams);
        
        if (empty($consumer["CDMUNICIPIO"]))
            $consumer["CDMUNICIPIO"] = $this->consumerFactory->getCityCode($orderData["address"]["state"], $orderData["address"]["city"]);

        if (empty($consumer["CDMUNICIPIO"]))
            $consumer["CDMUNICIPIO"] = $this->branchFactory->getBranchCityCode($generalParams["CDFILIAL"]);
            
        $consumer["DSAREAATEND"] = "";
        
        return array_merge($consumer, $address);
    }
    
    private function processAddressInformation ($orderData, $branchAddressInfo, $generalParams) {
        
        $consumer = null;
        
        $consumer["street"]          = $orderData["address"]["street"];
        $consumer["number"]          = $orderData["address"]["number"];
        $consumer["neighborhood"]    = $orderData["address"]["neighborhood"];
        $consumer["city"]            = $orderData["address"]["city"];
        $consumer["cep"]             = $orderData["address"]["zipcode"];
        $consumer["complement"]      = $orderData["address"]["complement"];
        $consumer["reference_point"] = $orderData["address"]["reference_point"];
        
        $consumer["CDMUNICIPIO"]     = $branchAddressInfo["CDMUNICIPIO"];
        $consumer["SGESTADO"]        = $branchAddressInfo["SGESTADO"];
        $consumer["CDBAIRRO"]        = $this->consumerFactory->getNeighborhoodCode($consumer);

        if ($orderData["type"] == "TAKEOUT") {
            $consumer["cep"]          = preg_replace("/[^0-9]/", "", $branchAddressInfo["NRCEPFILI"]);
            $consumer["street"]       = $branchAddressInfo["DSENDEFILI"];
            $consumer["neighborhood"] = $branchAddressInfo["NMBAIRFILI"];
        }
        
        return $consumer;
    }
    
    private function getDeliveryAreas($locationCoords) {
        
        $areas = $this->branchFactory->getDeliveryAreas();

        if(!$areas['error']) {
            $inside = [];

            $areas = $areas['areas'];
            foreach($areas as $cordinates) {
                $stringCoordinates   = $cordinates["COORDAREA"];
                $stringCoordinates   = str_replace("['","",$stringCoordinates);
                $stringCoordinates   = str_replace("']","",$stringCoordinates);
                $stringCoordinates   = str_replace("','",'$',$stringCoordinates);
                $polyCoordinates     = [];
                $explodedCoordinates = explode('$',$stringCoordinates);  
                
                foreach($explodedCoordinates as $latLongStr) {
                    $latLong = explode(',',$latLongStr);
                    array_push($polyCoordinates, $latLong);
                }

                $inside = $this->inside($locationCoords, $polyCoordinates);
               
                if($inside) {
                    $inside = [
                        "INSIDE"    => $inside,
                        "NAMEAREA"  => $cordinates["DSAREAATEND"],
                    ];

                    return $inside;    
                }
            }

            return $inside;
        } else {
            throw new \Exception($areas["message"], 992); 
        }
    }
    
    private function inside($point, $vs) {
        $x = (float)$point["lat"];
        $y = (float)$point["lng"];

        $inside = false;
        for ($i = 0, $j = count($vs) - 1; $i < count($vs); $j = $i++) {
            $xi = (float)$vs[$i][0];
            $yi = (float)$vs[$i][1];
            $xj = (float)$vs[$j][0];
            $yj = (float)$vs[$j][1];

            $intersect = (($yi > $y) != ($yj > $y)) && ($x < ($xj - $xi) * ($y - $yi) / ($yj - $yi) + $xi);
            if ($intersect) 
                $inside = !$inside;
        }
        
        return $inside;
    }
}
